local mod = ...



-- Default Balancing --------------------------------------------------------------------------------------------
local popInitial = 8

mod:overrideAsset({
    Id = "DEFAULT_BALANCING",
    InitialFamilyCount = popInitial,
    InitialGoldCapacity = 200,
    BuildingList = {},
    DelayedQuestList = {},
    MilitaryCampaignList = {
        "MILITARY_CAMPAIGN_EASY",
        "MILITARY_CAMPAIGN_AVERAGE",
        "MILITARY_CAMPAIGN_HARD",
    },
    VillagerCountPerImmigrationWave = {2, 2},
    EventList = {
        Action = "APPEND",
        "EVENT_IAE_BEGIN",
        "EVENT_IAE_MARKETGRANARY",
        "EVENT_IAE_HOUSING",
        "EVENT_IAE_LORDMANOR",
        "EVENT_IAE_CHURCH",
    },

})



-- Village Center inventory -------------------------------------------------------------------------------------
mod:overrideAsset({
	Id = "DEFAULT_VILLAGE_CENTER_INVENTORY",
	Collection = {
        -- Action = "APPEND",
        {Resource = "GOLD_COINS", Quantity = 200},
        {Resource = "WOOD", Quantity = 40},
        {Resource = "TOOL", Quantity = 40},
        {Resource = "PLANK", Quantity = 20},
        {Resource = "WOOLEN_CLOTH", Quantity = 20},
    }
})

-- Account for basic materials mod
if foundation.isModLoaded("d757c3ab-0148-4290-aa51-5559329f5640") then
    mod:overrideAsset({
        Id = "DEFAULT_VILLAGE_CENTER_INVENTORY",
        Collection = {
            Action = "APPEND",
            {Resource = "RESOURCE_SHINGLE", Quantity = 40},
            {Resource = "RESOURCE_TILE", Quantity = 40},
        }
    })
end



-- Building Progress List --------------------------------------------------------------------------------------------
local popUnlockMarket = 10
local popUnlockHouse = 10
local popUnlockManor = 16
local popUnlockChurch = 20


mod:overrideAsset({
    Id = "BUILDING_PROGRESS_LIST_DEFAULT",
    ProgressList = {
        {
            UnlockName = "",
            AssetBuildingList = {
                "BUILDING_IAE_HANDCART",
            },
            AssetUnlockableBuildingPartList = {},
            IsManuallyUnlocked = false,
            VillagerRequired = {
                Status = "NEWCOMER",
                Quantity = popInitial,
            },
        },
        {
            UnlockName = "UNLOCK_IAE_BUILDINGLIST1",
            AssetBuildingList = {
                "BUILDING_GATHERING_CAMP",
                "BUILDING_WOODCUTTER_CAMP",
                "BUILDING_STONECUTTER_CAMP",
                "BUILDING_HUNTERS_HUT",
                "BUILDING_FORESTER",
                "BUILDING_WELL",
                "MONUMENT_WOODEN_BRIDGE",
            },
            IsManuallyUnlocked = true,
            VillagerRequired = {
                Status = "NEWCOMER",
                Quantity = popInitial,
            }
        },
        {
            UnlockName = "UNLOCK_IAE_BUILDINGLIST2",
            AssetBuildingList = {
                "MONUMENT_MARKET",
                "BUILDING_GRANARY",
            },
            IsManuallyUnlocked = false,
            VillagerRequired = {
                Status = "NEWCOMER",
                Quantity = popUnlockMarket
            }
        },
        {
            UnlockName = "UNLOCK_IAE_BUILDINGLIST3",
            AssetBuildingList = {
                "BUILDING_HOUSE",
            },
            IsManuallyUnlocked = false,
            VillagerRequired = {
                Status = "NEWCOMER",
                Quantity = popUnlockHouse,
            }
        },
        {
            UnlockName = "UNLOCK_IAE_BUILDINGLIST4",
            AssetBuildingList = {
                "MONUMENT_RUSTIC_LORD_MANOR",
                "BUILDING_SAWMILL",
            },
            IsManuallyUnlocked = false,
            VillagerRequired = {
                Status = "NEWCOMER",
                Quantity = popUnlockManor,
            }
        },
        {
            UnlockName = "UNLOCK_IAE_BUILDINGLIST5",
            AssetUnlockableBuildingPartList = {
                "UNLOCKABLE_LABOUR_MARKET_GOODS_STALL",
            },
            IsManuallyUnlocked = false,
            VillagerRequired = {
                Status = "COMMONER",
                Quantity = 1,
            }
        },
        {
            UnlockName = "UNLOCK_IAE_BUILDINGLIST6",
            AssetUnlockableBuildingPartList = {
                "UNLOCKABLE_LABOUR_MARKET_LUXURY_STALL",
            },
            IsManuallyUnlocked = false,
            VillagerRequired = {
                Status = "CITIZEN",
                Quantity = 1,
            }
        },
    },
})



-- Building Requirements --------------------------------------------------------------------------------------------
local function removeBuildingReq(idBuilding)

    mod:overrideAsset({
        Id = idBuilding,
        VillagerRequired = nil,
        IsManuallyUnlocked = true,
    })

end

removeBuildingReq("BUILDING_BAKERY")
removeBuildingReq("BUILDING_BLACKSMITH")
removeBuildingReq("BUILDING_BREWERY")
removeBuildingReq("BUILDING_BUILDER_WORKSHOP")
removeBuildingReq("BUILDING_CHEESE_MAKER")
removeBuildingReq("BUILDING_COAL_HUT")
removeBuildingReq("BUILDING_COOPERAGE")
removeBuildingReq("BUILDING_DAIRY_FARM")
removeBuildingReq("BUILDING_DECORATIVE_BUSH")
removeBuildingReq("BUILDING_DECORATIVE_CYPRES")
removeBuildingReq("BUILDING_DECORATIVE_TREE")
removeBuildingReq("BUILDING_FENCE")
removeBuildingReq("BUILDING_FISHING_HUT")
removeBuildingReq("BUILDING_FORESTER")
removeBuildingReq("BUILDING_FOUNTAIN")
removeBuildingReq("BUILDING_GATHERING_CAMP")
removeBuildingReq("BUILDING_GLASS_SMELTER")
removeBuildingReq("BUILDING_GOLD_SMELTER")
removeBuildingReq("BUILDING_GRANARY")
removeBuildingReq("BUILDING_HOP_FARM")
removeBuildingReq("BUILDING_HOUSE")
removeBuildingReq("BUILDING_HUNTERS_HUT")
removeBuildingReq("BUILDING_IRON_SMELTER")
removeBuildingReq("BUILDING_JEWELER_WORKSHOP")
removeBuildingReq("BUILDING_KNIGHT_STATUE")
removeBuildingReq("BUILDING_MONK_STATUE")
removeBuildingReq("BUILDING_PLANT_ARCH")
removeBuildingReq("BUILDING_QUARRY_GOLD_DEPOSIT")
removeBuildingReq("BUILDING_QUARRY_IRON_DEPOSIT")
removeBuildingReq("BUILDING_QUARRY_MARBLE_DEPOSIT")
removeBuildingReq("BUILDING_QUARRY_QUARTZ_DEPOSIT")
removeBuildingReq("BUILDING_QUARRY_STONE_DEPOSIT")
removeBuildingReq("BUILDING_ROAD_SIGN")
removeBuildingReq("BUILDING_SAWMILL")
removeBuildingReq("BUILDING_SHEEP_FARM")
removeBuildingReq("BUILDING_SHEEP_STATUE")
removeBuildingReq("BUILDING_STONECUTTER_CAMP")
removeBuildingReq("BUILDING_STONEMASON_HUT")
removeBuildingReq("BUILDING_TAILOR_WORKSHOP")
removeBuildingReq("BUILDING_VILLAGE_CENTER")
removeBuildingReq("BUILDING_VINEYARD")
removeBuildingReq("BUILDING_WAREHOUSE")
removeBuildingReq("BUILDING_WEAPONSMITH")
removeBuildingReq("BUILDING_WEAVER_HUT")
removeBuildingReq("BUILDING_WELL")
removeBuildingReq("BUILDING_WHEAT_FARM")
removeBuildingReq("BUILDING_WINDMILL")
removeBuildingReq("BUILDING_WOODCUTTER_CAMP")
removeBuildingReq("BUILDING_WOODEN_GATE")
removeBuildingReq("MONUMENT_CHURCH")
removeBuildingReq("MONUMENT_LEGACY_RUSTIC_CHURCH")
removeBuildingReq("MONUMENT_MARKET")
removeBuildingReq("MONUMENT_MONASTERY")
removeBuildingReq("MONUMENT_ROMAN_CHURCH")
removeBuildingReq("MONUMENT_RUSTIC_CHURCH")
removeBuildingReq("MONUMENT_RUSTIC_LORD_MANOR")
removeBuildingReq("MONUMENT_STONE_BRIDGE")
removeBuildingReq("MONUMENT_TAVERN")
-- removeBuildingReq("MONUMENT_WOODEN_BRIDGE")
removeBuildingReq("MONUMENT_WOODEN_KEEP")



-- Certain mods that need to be delayed but will not be in the Estate menu ---------------------------------------------------------------------------------------------
local function delayMod(modGUID, nameBuilding, pop)
    if (foundation.isModLoaded(modGUID)) then
        mod:overrideAsset({
            Id = nameBuilding,
            VillagerRequired = { Status = "NEWCOMER", Quantity = pop },
        })
    end
end

delayMod("23b4f5f5-b7fd-4d79-80fd-d9943211c460", "BUILDING_EMPTY_HOUSE", popUnlockHouse )
delayMod("7786b0a3-cb8f-4c78-9ba6-ca3745b7c677", "JK_RESIDENTIAL_AREA", popUnlockHouse )
delayMod("af9c1101-b6f4-4067-b787-f2178db03739", "BUILDING_HOUSEPLOT", popUnlockHouse )
delayMod("d757c3ab-0148-4290-aa51-5559329f5640", "BUILDING_BASIC_CLAYPIT", popUnlockHouse )
delayMod("d757c3ab-0148-4290-aa51-5559329f5640", "BUILDING_BASIC_THATCHER", popUnlockHouse )



-- Event: Begin ---------------------------------------------------------------------------------------------
-- This begins the first quest
mod:registerAsset({
	DataType = "EVENT",
    Id = "EVENT_IAE_BEGIN",
    IsInMainPool = false,
    IsRecurrent = false,
    DaysBetweenOccurences = nil,
    DaysToFirst = 1,
    Delay = 1,
	ConditionList = {
		{
            DataType = "EVENT_CONDITION_BUILDING_BUILT",
            AssetBuilding = "BUILDING_VILLAGE_CENTER"
		}
	},
	ActionList = {
        {
            DataType = "EVENT_ACTION_GIVE_QUEST",
            Quest = "QUEST_IAE_HANDCART"
        },
        {
            DataType = "ACTION_SHOW_NARRATIVE_PANEL",
            NarrativePanelData = {
                Title = "EVENT_IAE_START_NAME",
                Description = "EVENT_IAE_START_DESC",
                PanelVisual = "ILLUSTRATION_ESTATE_LABOR",
            },
            ChoiceList = {
                {
                    ShortName = "TEXT_IAE_OK",
                },
            },
        },
    }
})



-- Quest: Handcart ---------------------------------------------------------------------------------------------
-- Handcart provides initial supplies to the villagers so that they don't need any other buildings to eat and drink for about 2 weeks
mod:registerAsset({
    DataType = "QUEST",
    Id = "QUEST_IAE_HANDCART",
    Name = "QUEST_IAE_HANDCART_NAME",
    Description = "QUEST_IAE_HANDCART_DESC",
    IsSilent = false,
	QuestFunction = {
        DataType = "QUEST_FUNCTION_CONTAINER",
        QuestContainerDescription = "QUEST_IAE_HANDCART_CONTAINER_DESC",
        SubQuestList = {
            {
                DataType = "QUEST_FUNCTION_BUILD",
                BuildingListToBuild = {
                    "BUILDING_IAE_HANDCART"
                }
            },
            {
                DataType = "QUEST_FUNCTION_ASSIGN_WORKER",
                Job = "BUILDER"
            },
        }
    },
    QuestRewardActionList = {
        {
            DataType = "QUEST_REWARD_UNLOCK_BUILDING_LIST",
            UnlockName = "UNLOCK_IAE_BUILDINGLIST1",
        },
    },
    QuestFailActionList = nil,
    QuestListToUnlockAfter = {
        {
            AssetQuest = "QUEST_IAE_IMMIGRATION",
            Delay = 5,
        },
    },
    OnFailCallback = nil,
    OnSuccessCallback = nil
})



-- Quest: Immigration ---------------------------------------------------------------------------------------------
-- Get the first immigrants
mod:registerAsset({
    DataType = "QUEST",
    Id = "QUEST_IAE_IMMIGRATION",
    Name = "QUEST_IAE_IMMIGRATION_NAME",
    Description = "QUEST_IAE_IMMIGRATION_DESC",
    IsSilent = false,
	QuestFunction = {
        DataType = "QUEST_FUNCTION_CONTAINER",
        QuestContainerDescription = "QUEST_IAE_IMMIGRATION_CONTAINER_DESC",
        SubQuestList = {
            {
                DataType = "QUEST_FUNCTION_ASSIGN_JOB_ALL_VILLAGERS",
            },
            {
                DataType = "QUEST_FUNCTION_MAX_HAPPINESS",
            },
        }
    },
    QuestRewardActionList = {
        {
            DataType = "QUEST_REWARD_IMMIGRATION_WAVE",
        },
        {
            DataType = "QUEST_REWARD_INFLUENCE",
            EstateInfluence = {
                Estate = "LABOUR",
                Quantity = 10,
            },
        },
    },
    QuestFailActionList = nil,
    QuestListToUnlockAfter = nil,
    OnFailCallback = nil,
    OnSuccessCallback = {
        DataType = "EVENT_CALLBACK_TRIGGER_EVENT",
        Event = "EVENT_IAE_IMMIGRATION",
    },
})

mod:registerAsset({
    DataType = "EVENT",
    Id = "EVENT_IAE_IMMIGRATION",
    Title = "EVENT_IAE_IMMIGRATION_NAME",
    Description = "EVENT_IAE_IMMIGRATION_DESC",
    ActionList = {
        {
            DataType = "ACTION_SHOW_NARRATIVE_PANEL",
            NarrativePanelData = {
                Title = "EVENT_IAE_IMMIGRATION_NAME",
                Description = "EVENT_IAE_IMMIGRATION_DESC",
                PanelVisual = "ILLUSTRATION_ESTATE_LABOR",
            },
            ChoiceList = {
                {
                    ShortName = "TEXT_IAE_OK",
                },
            },
        },
    },
})



-- Quest: Market and Granary ---------------------------------------------------------------------------------------------
mod:registerAsset({
    DataType = "EVENT",
    Id = "EVENT_IAE_MARKETGRANARY",
    Title = "EVENT_IAE_MARKETGRANARY_NAME",
    Description = "EVENT_IAE_MARKETGRANARY_DESC",
    ConditionList = {
        {
            DataType = "GAME_CONDITION_VILLAGER_STATUS_REACHED",
            AssetVillagerStatus = "NEWCOMER",
            NumberRequired = popUnlockMarket,
        },
    },
    ActionList = {
        {
            DataType = "ACTION_SHOW_NARRATIVE_PANEL",
            NarrativePanelData = {
                Title = "EVENT_IAE_MARKETGRANARY_NAME",
                Description = "EVENT_IAE_MARKETGRANARY_DESC",
                PanelVisual = "ILLUSTRATION_ESTATE_LABOR",
            },
            ChoiceList = {
                {
                    ShortName = "TEXT_IAE_OK",
                    EventActionList = {
                        {
                            DataType = "EVENT_ACTION_GIVE_QUEST",
                            Quest = "QUEST_IAE_MARKETGRANARY",
                        },
                    },
                },
            },
        },
    },
})

mod:registerAsset({
    DataType = "QUEST",
    Id = "QUEST_IAE_MARKETGRANARY",
    Name = "QUEST_IAE_MARKETGRANARY_NAME",
    IsSilent = false,
	QuestFunction = {
        DataType = "QUEST_FUNCTION_CONTAINER",
        QuestContainerDescription = "QUEST_IAE_MARKETGRANARY_CONTAINER_DESC",
        SubQuestList = {
            {
                DataType = "QUEST_FUNCTION_BUILD",
                BuildingListToBuild = {
                    "BUILDING_GRANARY",
                    "MONUMENT_MARKET",
                }
            },
        }
    },
    QuestRewardActionList = {
        {
            DataType = "GAME_ACTION_GIVE_RESOURCE_LIST",
            BudgetCategory = "QUESTS",
            ResourceCollection = {
                {
                    Resource = "GOLD_COINS",
                    Quantity = 50,
                },
            },
        },
    },
    QuestFailActionList = nil,
    QuestListToUnlockAfter = nil,
    OnFailCallback = nil,
    OnSuccessCallback = nil,
})



-- Event: Housing ---------------------------------------------------------------------------------------------
-- Notify the player that housing is unlocked and give instructions
mod:registerAsset({
    DataType = "EVENT",
    Id = "EVENT_IAE_HOUSING",
    DaysToFirst = 1,
    ConditionList = {
        {
            DataType = "GAME_CONDITION_VILLAGER_STATUS_REACHED",
            AssetVillagerStatus = "NEWCOMER",
            NumberRequired = popUnlockHouse,
        },
		{
            DataType = "GAME_CONDITION_QUEST_COMPLETED",
            Quest = "QUEST_IAE_MARKETGRANARY",
		},
    },
    ActionList = {
        {
            DataType = "ACTION_SHOW_NARRATIVE_PANEL",
            NarrativePanelData = {
                Title = "EVENT_IAE_HOUSING_NAME",
                Description = "EVENT_IAE_HOUSING_DESC",
                PanelVisual = "ILLUSTRATION_ESTATE_LABOR",
            },
            ChoiceList = {
                {
                    ShortName = "TEXT_IAE_OK",
                },
            },
        },
    },
})



-- Event: Lord Manor ---------------------------------------------------------------------------------------------
-- Start a quest to build the Lord Manor and promote villagers
mod:registerAsset({
    DataType = "EVENT",
    Id = "EVENT_IAE_LORDMANOR",
    DaysToFirst = 1,
    ConditionList = {
        {
            DataType = "GAME_CONDITION_VILLAGER_STATUS_REACHED",
            AssetVillagerStatus = "NEWCOMER",
            NumberRequired = popUnlockManor,
        },
    },
    ActionList = {
        {
            DataType = "ACTION_SHOW_NARRATIVE_PANEL",
            NarrativePanelData = {
                Title = "EVENT_IAE_LORDMANOR_NAME",
                Description = "EVENT_IAE_LORDMANOR_DESC",
                PanelVisual = "ILLUSTRATION_ESTATE_KING",
            },
            ChoiceList = {
                {
                    ShortName = "TEXT_IAE_OK",
                    EventActionList = {
                        {
                            DataType = "EVENT_ACTION_GIVE_QUEST",
                            Quest = "QUEST_IAE_LORDMANOR",
                        },
                    },
                },
            },
        },
    },
})

mod:registerAsset({
    DataType = "QUEST",
    Id = "QUEST_IAE_LORDMANOR",
    Name = "QUEST_IAE_LORDMANOR_NAME",
    Description = "QUEST_IAE_LORDMANOR_DESC",
    IsSilent = false,
	QuestFunction = {
        DataType = "QUEST_FUNCTION_CONTAINER",
        QuestContainerDescription = "QUEST_IAE_LORDMANOR_CONTAINER_DESC",
        SubQuestList = {
            {
                DataType = "QUEST_FUNCTION_BUILD",
                BuildingListToBuild = {
                    "MONUMENT_RUSTIC_LORD_MANOR",
                },
            },
            {
                DataType = "QUEST_FUNCTION_ASSIGN_BUILDING_FUNCTION",
                AssetBuilding = "MONUMENT_RUSTIC_LORD_MANOR",
                AssetBuildingFunction = "BUILDING_FUNCTION_GREAT_HALL",
            },
            {
                DataType = "QUEST_FUNCTION_REACH_VILLAGER_STATUS",
                StatusTargeted = "SERF",
            },
        },
    },
    QuestRewardActionList = {
        {
            DataType = "QUEST_REWARD_INFLUENCE",
            EstateInfluence = {
                Estate = "KINGDOM",
                Quantity = 10,
            },
        },
    },
    QuestFailActionList = nil,
    QuestListToUnlockAfter = nil,
    OnFailCallback = nil,
    OnSuccessCallback = nil,
})



-- Event: Church ---------------------------------------------------------------------------------------------
-- Start a quest to build a church
mod:registerAsset({
    DataType = "EVENT",
    Id = "EVENT_IAE_CHURCH",
    DaysToFirst = 1,
    ConditionList = {
        {
            DataType = "GAME_CONDITION_VILLAGER_STATUS_REACHED",
            AssetVillagerStatus = "NEWCOMER",
            NumberRequired = popUnlockChurch,
        },
    },
    ActionList = {
        {
            DataType = "ACTION_SHOW_NARRATIVE_PANEL",
            NarrativePanelData = {
                Title = "EVENT_IAE_CHURCH_NAME",
                Description = "EVENT_IAE_CHURCH_DESC",
                PanelVisual = "ILLUSTRATION_ESTATE_CHURCH",
            },
            ChoiceList = {
                {
                    ShortName = "TEXT_IAE_OK",
                    EventActionList = {
                        {
                            DataType = "EVENT_ACTION_GIVE_REWARD",
                            ActionList = {
                                {
                                    DataType = "QUEST_REWARD_INFLUENCE",
                                    EstateInfluence = {
                                        Estate = "CLERGY",
                                        Quantity = 10,
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
    },
})



-- Building Function overrides -------------------------------------------------------------------------------------
-- Reduce the number of workers in basic workplaces
local function overrideBasicWorkplace(func, cap)
    mod:overrideAsset({
        Id = func,
        WorkerCapacity = cap,
    })
end

overrideBasicWorkplace("BUILDING_FUNCTION_GATHERING_HUT", 2)
overrideBasicWorkplace("BUILDING_FUNCTION_HUNTERS_HUT", 2)
overrideBasicWorkplace("BUILDING_FUNCTION_BUILDER_WORKSHOP", 2)
overrideBasicWorkplace("BUILDING_FUNCTION_STONECUTTER_CAMP", 2)
overrideBasicWorkplace("BUILDING_FUNCTION_WOODCUTTER", 2)

if (foundation.isModLoaded("35f7b4b6-105d-4377-a727-5ea2997e556d")) then -- Fire mod
    overrideBasicWorkplace("BUILDING_FUNCTION_FIRE_WATER", 2)
end
